
typedef unsigned char UCHAR;

const UCHAR EP_NONE            = 0 ; // no program loaded
const UCHAR EP_CTRLBRK         = 1 ; // control-break from user program
const UCHAR EP_TRACE           = 2 ; // trace interrupt
const UCHAR EP_BKPT            = 3 ; // breakpoint interrupt
const UCHAR EP_TERM            = 4 ; // program terminated
const UCHAR EP_LOADED          = 5 ; // loaded - not run
const UCHAR EP_RUNNING         = 6 ; // user program running
const UCHAR EP_SWAPPED         = 7 ; // swapped to disk
const UCHAR EP_STEP            = 8 ; // step (trace/skip calls) interrupt
const UCHAR EP_KEYINTR         = 10; // keyboard interrupt (alt-break) etc.
const UCHAR EP_SYSBKPT         = 11; // system breakpoint interrupt
const UCHAR EP_GLOBAL          = 12; // global breakpoint interrupt
const UCHAR EP_NMI             = 13; // got an nmi
const UCHAR EP_HARDBKP         = 14; // interrupt from hardware bkpt
const UCHAR EP_EXCEPTION       = 16; // OS/2 protection violation
const UCHAR EP_FPERROR         = 17; // floating point error
const UCHAR EP_ALLFROZEN       = 18; // all threads frozen
const UCHAR EP_THREADTERM      = 19; // a thread has terminated
const UCHAR EP_SIGNAL          = 20; // signal about to be received by child
const UCHAR EP_NEWPROCESS      = 21; // new process started
const UCHAR EP_WINBKPT         = 22; // windows message on winproc breakpoint
const UCHAR EP_EXCEPFIRSTTRY   = 23; // stop on first debugger notification of exception
const UCHAR EP_EXCEPUNEXPECTED = 24; // stop on unexpected C++ exception
const UCHAR EP_EXCEPTERMINATE  = 25; // stop on C++ exception handling error
const UCHAR EP_SYSERROR        = 27; // Windows NT system error

// struct coproc uses the following types

typedef struct
{
  char signif[8];
  char exp[2];
} REG87;

struct c   //  control flags
{
  unsigned im:1;
  unsigned dm:1;
  unsigned zm:1;
  unsigned om:1;
  unsigned um:1;
  unsigned pm:1;
  unsigned fill1:1;
  unsigned iem:1;
  unsigned pc:2;
  unsigned rc:2;
  unsigned ic:1;
  unsigned fill2:3;
};

struct s
{               // status flags
  unsigned ie:1;
  unsigned de:1;
  unsigned ze:1;
  unsigned oe:1;
  unsigned ue:1;
  unsigned pe:1;
  unsigned ir:1;
  unsigned fill:1;
  unsigned cc02:3;
  unsigned st:3;
  unsigned cc3:1;
  unsigned b:1;
};

struct t
{               // register tags
  unsigned tag0:2;
  unsigned tag1:2;
  unsigned tag2:2;
  unsigned tag3:2;
  unsigned tag4:2;
  unsigned tag5:2;
  unsigned tag6:2;
  unsigned tag7:2;
};

struct i
{               // instruction/operand pointer
  unsigned offst:16;
  unsigned opcode:12;
  unsigned seg:4;
};
struct cp87
{
  struct c cn;
  struct s st;
  struct t tg;
  struct i ip;
  struct i op;
  REG87 reg[8];
};

struct i387
{            // instruction/operand pointer
  unsigned long offst;
  unsigned short selector;
  unsigned short opcode;
};

struct coproc
{
  struct c cn;
  unsigned short filler1;
  struct s st;
  unsigned short filler2;
  struct t tg;
  unsigned short filler3;
  struct i387 ip;
  struct i387 op;
  REG87 reg[8];
};

// struct cpu has the following layout:

typedef unsigned long PROGOFFSET;

typedef struct
{
  PROGOFFSET offset;
  unsigned short segment;
} PROGADDR;

// status flags structure

struct f
{
  unsigned cf:1;      // carry flag
  unsigned :1;
  unsigned pf:1;      // parity flag
  unsigned :1;
  unsigned af:1;      // auxiliary carry flag
  unsigned :1;
  unsigned zf:1;      // zero flag
  unsigned sf:1;      // sign flag
  unsigned tf:1;      // trap flag flag
  unsigned ef:1;      // interrupt enable flag
  unsigned df:1;      // direction flag
  unsigned of:1;      // overflow flag
  unsigned fill:4;
  unsigned fill2:16;
};

struct cpu
{
  union
  {
    struct wr
    {
      unsigned long uax;       // 32-bit registers
      unsigned long ubx;
      unsigned long ucx;
      unsigned long udx;
    } w;

    struct br
    {
      unsigned char al, ah, a_32[2];  // byte registers
      unsigned char bl, bh, b_32[2];
      unsigned char cl, ch, c_32[2];
      unsigned char dl, dh, d_32[2];
    } b;
  } regs;

  unsigned long usp;            // stack pointer
  unsigned long ubp;            // base pointer
  unsigned long usi;            // source index
  unsigned long udi;            // destination index
  struct f fl;                  // status flags
  PROGADDR csip;                // instruction pointer
  unsigned short uds;           // data segment
  unsigned short uss;           // stack segment
  unsigned short ues;           // extra segment
  unsigned short ufs;           // 386 fs register
  unsigned short ugs;           // 386 gs register
};

typedef struct
{
  short InfoBlockSize;            // so we can add fields at the end
  unsigned short APIVersion;      // 16-bit version number for this API

  // useful function pointers inside TD

    // evaluate expression and return result
    int (*fpEvalFunc)(char *evalstring, char *resultstring);

  // useful data pointers inside TD

    struct cpu *pCPU;             // ptr to current CPU state block
    struct coproc *p8087;         // ptr to current numeric processor state block
    unsigned char *pProgState;    // ptr to user program state
    #ifdef __FLAT__
      unsigned *pCurrentThreadID; // ptr to current thread id
      unsigned *pUserProcessID;   // ptr to user process id
    #endif
    unsigned short *pScreenSel;   // ptr to selector used for debugger screen
} TDUserInfoBlock;

#define _WIN __export __pascal

extern "C" {
int   _WIN UserAPIInit (TDUserInfoBlock *p);
void  _WIN UserAPIDone (void);
void  _WIN UserAPIModuleLoad (char *modname, long handle);
void  _WIN UserAPIModuleUnload (char *modname, long handle);
unsigned short _WIN UserAPIKeyPressed (unsigned short bioskeycode);
unsigned short _WIN UserAPIPollForKey (int biosmode);
void  _WIN UserAPIChildStop (void);
void  _WIN UserAPIChildStart (void);
}