/*
  UICTL.CPP
  Main source file for class UIInputLine, UIInputShort, UIInputLong,
                             UISpinner, UIMsgBox

  Copyright (c) 1994 by Borland International
  All Rights Reserved

  This source code is hereby released into the public domain.
  Written by Jeffrey J. Peters

*/

#include <ctype.h>
#include "uiclass.h"

bits32 Extract (char *s);

//----------------------------------------------------------------------------

UIInputLine::UIInputLine (UIWindow &w, bits16 len, char *title) : parent (w),
                          maxLength(len), text (0),
                          UIWindow (UIRect(w.rFrame.ul.x+2,w.rFrame.ul.y+2,
                                           w.rFrame.ul.x+7+len,w.rFrame.ul.y+7),
                                    title)
{
  uiFlags |= UIFShowCursor;
  uiFlags |= UIFNoScroll;
  uiFlags |= UIFNoStretch;
  uiFlags |= UIFNoZoom;
  text = new char [maxLength];
  text[0] = 0;
  dispLen = -1;

  if (w.rFrame.ul.y > 15)
    MoveTo (UIRect (rFrame)+= UIRect (0,-7,0,-7));
}

int UIInputLine::Update (int dsp)
{
  textattr (WHITE|_BLUE);

  gotoxy (2,2);
  if ((strlen (text) < dispLen) || (dsp == 1))
  {
    for (int j=0;j<maxLength;j++)
    {
      gotoxy (2+j,2);
      putch (' ');
    }
    gotoxy (2,2);
  }
  if (text[0])
    cputs (text);
  else
  {
    cputs (" ");
    gotoxy (2,2);
  }
  dispLen = strlen (text);
  return 1;
}

char *UIInputLine::begin (void)
{
  int out = 0, l;
  bits16 c;
  char str[2] = {0,0};

  SetLineType (1);
  DrawBox (uiFlags, WHITE|_LIGHTGRAY, WHITE|_LIGHTGRAY);

  while (!out)
  {
    Redraw (0);
    switch (c = getch())
    {
      case K_F1:
        {
          UIMsgBox (
          " <Esc>         - return to previous window\r\n"
          " <Ctrl-F5>     - move window\r\n"
          " <Enter>       - return value\r\n"
          "\r\n"
          "Use registers (ie: ax or _EBP), variables or\r\n"
          "hex based numbers (12DB or abcd).\r\n"
          "For searching, use (05 EA 23 D5) or ('Borland')."
          ,  " Input Help  " ,7, 60);
        }
        break;

      case K_ESC:
        out = 1;
        delete text;
        text = 0;
        break;

      case K_ENTER:
        out = 1;
        return text;

      case K_DEL:
      case K_BS:
        if (strlen (text))
          text[strlen(text)-1] = 0;
        Redraw (0);
        break;

      default:
        if (isprint (c&0x00FF))
        {
          l = strlen (text);
          str[0] = c&0x00FF;
          if (l < maxLength)
          {
            strcat (text, str);
            Redraw (0);
          }
        }
        break;
    }
  }
  return NULL;
}
//----------------------------------------------------------------------------
bits32 Extract (char *s)
{
  char *start, *end;
  bits32 data;
  start = s;

  while (1)
  {
    while (start[0] == ' ') start++;

    if (isdigit(start[0]))
      break;

    while (start[0] != ' ')
      if (start[0] != 0)
        start++;
      else
        return 0;
  }
  end = start;
  while (isdigit(end[0])) end++;

  end[0] = 0;

  data = strtoul (start, NULL, 10);
  return data;
}
//----------------------------------------------------------------------------
bits16 UIInputShort::Value (bits16 dflt)
{
  char *p;
  char *q;
  char dest[100];

  p = UIInputLine::begin ();
  if (p && p[0])
  {
    EvalExpr (p, dest);
    data = (bits16) Extract (dest);
    delete p;
  }
  else
    data = dflt;

  return data;
}
//----------------------------------------------------------------------------
bits32 UIInputLong::Value (bits32 dflt)
{
  char *p;
  char *q;
  char dest[100];

  p = UIInputLine::begin ();
  if (p && p[0])
  {
    EvalExpr (p, dest);
    data = (bits32) Extract (dest);
    delete p;
  }
  else
    data = dflt;

  return data;
}
//----------------------------------------------------------------------------

UIMsgBox::UIMsgBox (char *string, char *title, int lns, int wid) : text (0),
                    UIWindow (UIRect(40-(wid/2),12-(lns/2),
                              40+(wid/2),14+(lns/2)),title)
{
  text = new char [strlen(string)+1];
  strcpy (text, string);

  uiFlags |= UIFNoScroll;
  uiFlags |= UIFNoStretch;
  uiFlags |= UIFNoZoom;
  uiFlags |= UIFNoCursorFix;

  SetLineType (1);
  DrawBox (uiFlags, BLACK|_LIGHTGRAY, WHITE|_LIGHTGRAY);
  getch ();
}
UIMsgBox::~UIMsgBox ()
{
  if (text)
    delete text;
}
#pragma argsused
int UIMsgBox::Update (int dsp)
{
  gotoxy (1,1);
  if (text)
    cputs (text);
  return 1;
}
//----------------------------------------------------------------------------
void UISpinner::Spin (int dir)
{
  static char ptr = 0;
  static char string[] = "\\|/-";
  int x,y;

  x = wherex();
  y = wherey();

  {
    UIRect &r = parent.rFrame;
    window (r.ul.x, r.ul.y, r.lr.x, r.lr.y);
  }

  gotoxy (2,1);
  if (!hold)
    hold = GetCharAt (2,1);

  putch (string[ptr]);

  if (dir)
  {
    ptr++;
    if (ptr > 3)
      ptr = 0;
  }
  else
  {
    if (ptr == 0)
      ptr = 3;
    else
      ptr--;
  }
  {
    UIRect &r = parent.rWindow;
    window (r.ul.x, r.ul.y, r.lr.x, r.lr.y);
  }
  gotoxy (x,y);
}

void UISpinner::Restore (void)
{
  {
    UIRect &r = parent.rFrame;
    window (r.ul.x, r.ul.y, r.lr.x, r.lr.y);
  }
  if (hold)
    PutCharAt (2,1, hold);
  hold = 0;
}
//----------------------------------------------------------------------------


