/*
  UISEL.CPP
  Main source file for class UISel


  Copyright (c) 1994 by Borland International
  All Rights Reserved

  This source code is hereby released into the public domain.
  Written by Jeffrey J. Peters

*/

#include <ctype.h>
#include <alloc.h>
#include <dos.h>
#include <new.h>
#include "uiclass.h"

#define INCREMENT 8

UISel::UISel (UIRect r, char *title) : UIWindow (r,title)
{
  lineIndex = 0;
#if defined (__DPMI16__) || defined (__DPMI32__)
  sel = 0x7;
#else
  sel = 0;
#endif
  uiFlags |= UIFNoScroll;
}


#ifdef __DPMI16__
typedef short far __pascal (*Tpfn_i_pi_pc) (short,short*,char*);
char *GetMod (bits16 sel)
{
  static HMODULE hRTM = 0;
  static Tpfn_i_pi_pc pfnMODSel2Seg = 0;
  static char buf[81];
  char temp[81];
  HMODULE mod;
  bits08 seg;
  bits16 ret;

  strcpy (buf, "<Unknown>");

  if (!IsValidSel (sel))
    return buf;

  sel &= (~0x8);  // RTM needs to see the bottom bits off for a selector

  if (!hRTM)
  {
    hRTM = GetModuleHandle ("RTM.EXE");
      if (!hRTM)
        return buf;
  }
  if (!pfnMODSel2Seg)
  {
    pfnMODSel2Seg = (Tpfn_i_pi_pc) GetProcAddress (hRTM, (char*)MK_FP (0,43));
    if (!pfnMODSel2Seg)
      return buf;
  }

  ret = pfnMODSel2Seg (sel, (short*)&mod, (char*)&seg);
  if (ret == 0)
  {
    GetModuleFileName (mod, temp, 60);
    sprintf (buf, "#%d:%s",(int)seg, temp);
  }

  return buf;
}
#else
#pragma argsused
char *GetMod (bits16 sel)
{
  return "<Unknown>";
}
#endif

#if defined (__DPMI32__) || (__DPMI16__)
char *UISel::Calculate (bits16 s)
{
  static char buf[80];
  bits32 base,limit;
  bits08 rights;

  base = GetBase (s);
  if (base == (bits32)-1)
  {
    sprintf (buf, "%04X:<Invalid Selector>", s);
    return buf;
  }

  limit = GetLimit (s);
  rights = GetRights (s);
#ifdef __DPMI16__
  char *mod;
  mod = GetMod (s);
  sprintf (buf, "%04X:[%08lX](%08lX)<%02X> %s", s, base,limit,rights, mod);
#else
  sprintf (buf, "%04X:[%08lX](%08lX)<%02X> ", s, base,limit,rights);
#endif
  return buf;
}
#else
char *UISel::Calculate (bits16 s)
{
  static char buf[80];
  sprintf (buf, "%04X:<No selectors in real-mode>", s);
  return buf;
}
#endif

#pragma argsused
int UISel::Update (int dsp)
{
  int j;
  char *buf;

  if (lineIndex > rWindow.Height())
    lineIndex = rWindow.Height()-1;

  for (j=0;j<rWindow.Height();j++)
  {
    buf = Calculate (sel+(j*INCREMENT));
    if (strlen (buf) >= rWindow.Width())
      buf[rWindow.Width()] = 0;
    gotoxy (1,j+1);
    if (j == lineIndex)
    {
      textattr (YELLOW|_BLUE);
      clreol ();
      cputs (buf);
      textattr (BLACK|_CYAN);
    }
    else
    {
      clreol ();
      cputs (buf);
    }
  }
  return 1;
}

int UISel::begin (void)
{
  int out = 0, cx,cy;
  char *p;
  UIInputShort *il;
  UIDmp *dmp;
  UISelView *uisv;

  SetLineType (1);
  DrawBox (uiFlags, BLACK|_CYAN, WHITE|_CYAN);

  Redraw (0);
  while (!out)
  {
    while (kbhit ())
      getch ();            // suck up extra keys

    switch (getch ())
    {
      case K_F1:
        {
          UIMsgBox (
          " <Esc>         - exit to previous window\r\n"
          " <Up><Down>,   \r\n"
          " <PgUp><PgDwn> - moves the highlite bar around\r\n"
          " <F5>          - zooms the screen\r\n"
          " <Ctrl-F5>     - move/re-size the window\r\n"
          " <Ctrl-D>      - detailed info on the current selector\r\n"
          " <Ctrl-S>      - enter selector to view\r\n"
          " <Enter>       - view dump of current selector\r\n"
          " <Shft-Up>,    \r\n"
          " <Shft-Dwn>    - adjust selector by +-1"
          ,  " Selector Window Help " ,10, 60);
        }
        break;

      case K_ESC:
        out = 1;
        break;

      case K_CTRL_HOME:
        lineIndex = 0;
        sel = 7;
        Redraw (0);
        break;

      case K_CTRL_ENTER:
      case K_CTRL_D:
      #if defined(__DPMI16__) || defined (__DPMI32__)
        uisv = new UISelView (sel + (INCREMENT * lineIndex));
        uisv->begin ();
        delete uisv;
        Redraw (0);
      #endif
        break;

      case K_ENTER:
        dmp = new UIDmp (*this, (sel + (INCREMENT * lineIndex)), 0x0);
        dmp->begin ();
        delete dmp;
        Redraw (0);
        break;

      case K_CTRL_S:
        il = new UIInputShort (*this, " Enter a Selector ");
        sel = il->Value (sel);
        lineIndex = 0;
        delete il;
        Redraw (0);
        break;

      case K_UP:
        if (lineIndex == 0)
          sel -= INCREMENT;
        else
          lineIndex--;
        Redraw (0);
        break;

      case K_DOWN:
        if (lineIndex == rWindow.Height()-1)
          sel += INCREMENT;
        else
          lineIndex++;
        Redraw (0);
        break;

      case K_PGUP:
        sel -= (INCREMENT * rWindow.Height());
        Redraw (0);
        break;

      case K_PGDOWN:
        sel += (INCREMENT * rWindow.Height());
        Redraw (0);
        break;

      case K_CTRL_PGUP:
        sel -= (0x100);
        Redraw (0);
        break;

      case K_CTRL_PGDOWN:
        sel += (0x100);
        Redraw (0);
        break;

      case K_SHIFT_UP:
        sel -= 1;
        Redraw (0);
        break;

      case K_SHIFT_DOWN:
        sel += 1;
        Redraw (0);
        break;
    }
  }
  return 0;
}

/* Override the new and delete operators so that execptions don't get
   thrown.
*/
extern FILE *log;
extern int log_memory;
void *operator new (size_t sz)
{
  void *p = malloc (sz);
  if (log_memory && log)
  {
    #ifdef __DPMI32__
      fprintf (log, "Malloc: size:%04X  ptr:%p\r\n", sz,p);
    #else
      fprintf (log, "Malloc: size:%04X  ptr:%p  free:%08lX\r\n", sz,p,farcoreleft());
    #endif
    fflush (log);
  }
  assert (p);
  return p;
}
void operator delete (void *p)
{
  assert (p);
  free (p);

  if (log_memory && log)
  {
    #ifdef __DPMI32__
      fprintf (log, "Free:              ptr:%p\r\n",p);
    #else
      fprintf (log, "Free:              ptr:%p  free:%08lX\r\n",p,farcoreleft());
    #endif

    fflush (log);
  }
}
#pragma argsused
extern "C" void UIStart (bits16 screen, bits32 data)
{
  int cx,cy;
  cx = wherex ();
  cy = wherey ();

  #ifdef __DPMI32__
    UIObject::hPIDUser = (HANDLE)data;
  #endif
  UIObject::screenSel = screen;

  UISel &w = *new UISel (UIRect (2,12,80,15), " Selector Window ");
  w.begin ();

  delete &w;
  window (1,1,80,25);
  gotoxy (cx,cy);
}
